<?php
/**
 * $Horde: accounts/lib/Driver/ldap.php,v 1.12 2005/01/03 14:34:44 jan Exp $
 *
 * Copyright 2001-2005 Eric Rostetter <eric.rostetter@physics.utexas.edu>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * The ldap class attempts to return user information stored in an ldap
 * directory service.
 *
 * @author   Eric Jon Rostetter <eric.rostetter@physics.utexas.edu>
 * @version  $Revision: 1.12 $
 * @since    Accounts 2.1
 * @package  accounts
 */
class Accounts_Driver_ldap extends Accounts_Driver {

    /**
     * Pointer to the ldap connection.
     *
     * @var resource $_ds
     */
    var $_ds;

    /**
     * Hash containing connection parameters.
     *
     * @var array $_params
     */
    var $_params;

    /**
     * Constructs a new Accounts_Driver_ldap object.
     *
     * @param array $params  A hash containing connection parameters.
     */
    function Accounts_Driver_ldap($params = array())
    {
        $this->_params = array_merge(
            array('host' => 'localhost',
                  'port' => 389,
                  'basedn' => '',
                  'attr' => 'uid'),
            $params);
    }


    /**
     * Returns the user account from the ldap source.
     *
     * @return array  A hash with complete account details.
     */
    function _getAccount()
    {
        static $information;

        if (!isset($information)) {
            $username  = $this->getUsername();

            // connect to the ldap server
            $this->_ds = ldap_connect($this->_params['host'],
                                      $this->_params['port']);
            if (!$this->_ds) {
                return PEAR::raiseError(_("Could not connect to ldap server"));
            }

            // bind as anonymous
            $result = @ldap_bind($this->_ds);
            if (!$result) {
                return PEAR::raiseError(_("Could not bind to ldap server"));
            }

            // Get the fullname
            $searchResult = ldap_search($this->_ds, $this->_params['basedn'],
                                        $this->_params['attr'] . '=' . $username);
            $information = ldap_get_entries($this->_ds, $searchResult);

            // disconnect from the ldap server
            @ldap_close($this->_ds);

            if ($information['count'] == 0) {
                return PEAR::raiseError(_("User account not found"));
            }
        }

        return $information;
    }

    /**
     * Returns the user's full name.
     *
     * @return mixed  The user's full name (string), or false (error).
     */
    function getFullname()
    {
        $information = $this->_getAccount();

        if (isset($information[0]['cn;lang-es'][0]) &&
            $information[0]['cn;lang-es'][0] != '') {
            $name = $information[0]['cn;lang-es'][0];
        } else {
            $name = $information[0]['cn'][0];
        }

        return (empty($name) ? false : $name);
    }

    /**
     * Returns the user's home (login) directory.
     *
     * @return mixed  The user's directory (string), or false (error).
     */
    function getHome()
    {
        $information = $this->_getAccount();
        $homedir = $information[0]['homedirectory'][0];
        return (empty($homedir) ? false : $homedir);
    }

    /**
     * Returns the user's default shell.
     *
     * @return mixed  The user's shell (string), or false (error).
     */
    function getShell()
    {
        $information = $this->_getAccount();
        $shell = $information[0]['loginshell'][0];
        return (empty($shell) ? false : $shell);
    }

    /**
     * Returns the date of the user's last password change.
     *
     * @return mixed  Date string (string) or false (error).
     */
    function getPasswordChange()
    {
        $information = $this->_getAccount();

        if (isset($information[0]['shadowlastchange'][0])) {
            $lastchange = strftime('%x', $information[0]['shadowlastchange'][0] * 86400);
        }

        return (empty($lastchange) ? false : $lastchange);
    }

    /**
     * Returns the status of the current password.
     *
     * @return mixed  A string with a warning message if the password is about
     *                to expire, PEAR_Error on error and false otherwise.
     */
    function checkPasswordStatus()
    {
        $information = $this->_getAccount();

        if (is_a($information,'PEAR_Error')) {
            return $information;
        } else {
            if (isset($information[0]['shadowmax'][0]) &&
                isset($information[0]['shadowlastchange'][0]) &&
                isset($information[0]['shadowwarning'][0])) {
                $today = floor(time() / 86400);
                $warnday = $information[0]['shadowlastchange'][0] +
                           $information[0]['shadowmax'][0] - $information[0]['shadowwarning'][0];
                $toexpire = $information[0]['shadowlastchange'][0] + $information[0]['shadowmax'][0] - $today;

                if ($today >= $warnday) {
                    return sprintf(_("%d days until your password expires."), $toexpire);
                } else {
                    return false;
                }
            } else {
                return false;
            }
        }
    }

}
